@@ -19,10 +19,10 @@ from typing import (
 class Dependency(NamedTuple):
     tensor: Tensor
     grad_fn: Callable[[np.ndarray, Optional[Dict]], np.ndarray]
-    meta: Dict = None
+    meta: Optional[Dict] = None
 
 
-Shapable = Union[Tuple, List, int]
+Shapable = Union[Tuple[int], List[int], int]
 Arrayable = Union[float, list, np.ndarray]
 Tensorable = Union['Tensor', float, np.ndarray]
 
@@ -42,7 +42,9 @@ def ensure_tensor(tensorable: Tensorable) -> Tensor:
 
 
 class Tensor:
-    def __init__(self, data: Arrayable, requires_grad: bool = False, dependencies: List[Dependency] = None) -> None:
+    def __init__(self, data: Arrayable,
+                 requires_grad: bool = False,
+                 dependencies: List[Dependency] = None) -> None:
         self._data = ensure_array(data)
         self.requires_grad = requires_grad
         self.dependencies = dependencies or []
@@ -65,6 +67,14 @@ class Tensor:
     def numpy(self) -> np.ndarray:
         return self.data
 
+    def item(self):
+        if self.shape == ():
+            return self.data
+        elif self.shape == (1,):
+            return self.data[0]
+        else:
+            raise RuntimeError('must be one element')
+
     def zero_grad(self) -> None:
         self.grad = Tensor(np.zeros_like(self.data))
 
@@ -92,28 +102,39 @@ class Tensor:
         """the product of the self.datasets’s dimensions."""
         return self._data.size
 
-    def sum(self, axis: Optional[Shapable] = None) -> Tensor:
-        return sum(self, axis)
+    def sum(self, dim: Optional[Shapable] = None) -> Tensor:
+        return sum(self, dim)
 
-    def mean(self, axis: Optional[Shapable] = None) -> Tensor:
-        return mean(self, axis)
+    def mean(self, dim: Optional[Shapable] = None) -> Tensor:
+        return mean(self, dim)
 
     def transpose(self, dim1: int, dim2: int) -> Tensor:
         return transpose(self, dim1, dim2)
 
-    def permute(self, axes: Union[List, Tuple]) -> Tensor:
-        return permute(self, axes)
+    def permute(self, dims: Union[List, Tuple]) -> Tensor:
+        return permute(self, dims)
 
-    def reshape(self, *sp):
+    def reshape(self, *sp: int) -> Tensor:
         return reshape(self, sp)
 
     def flatten(self, start_dim=0, end_dim=-1):
         return flatten(self, start_dim, end_dim)
 
+    def diag(self, diagonal: int = 0) -> Tensor:
+        return diag(self, diagonal)
+
+    def index_select(self, dim: int, index) -> Tensor:
+        return index_select(self, dim, index)
+
     @property
     def T(self) -> Tensor:
         return permute(self, list(range(len(self.shape) - 1, -1, -1)))
 
+    def detach(self) -> Tensor:
+        new_data = self._data.copy()
+        new_requires_grad = self.requires_grad
+        return Tensor(new_data, new_requires_grad)
+
     def __add__(self, other) -> Tensor:
         """called if `self + other`"""
         return add(self, ensure_tensor(other))
@@ -183,9 +204,9 @@ class Tensor:
         self.data = self.data @ ensure_tensor(other).data
         return self
 
-    def __pow__(self, n):
+    def __pow__(self, e: Tensor) -> Tensor:
         """called if `self ** other`"""
-        return power(self, ensure_tensor(n))
+        return power(self, ensure_tensor(e))
 
     def __eq__(self, other: object) -> bool:
         """test if `self == other`"""
@@ -201,15 +222,32 @@ class Tensor:
     def __repr__(self) -> str:
         return f"Tensor({self.data}, shape={self.shape}, requires_grad={self.requires_grad})"
 
-    def __hash__(self):
+    def __hash__(self) -> int:
         return hash(str(self.data.data))
 
 
+def tensor(data: Arrayable, requires_grad: bool = False, dtype=None) -> Tensor:
+    if type(data).__module__ == np.__name__:
+        return Tensor(data.copy(), requires_grad=requires_grad)
+    return Tensor(data, requires_grad=requires_grad)
+
+
+def as_tensor(data: Arrayable, requires_grad: bool = False) -> Tensor:
+    return Tensor(data, requires_grad=requires_grad)
+
+
+def stack(tensors: List[Tensor], dim: int = 0) -> Tensor:
+    tensors = [unsqueeze(i, dim) for i in tensors]
+    return concat(tensors, dim)
+
+
 def zeros(shape, dtype=None, order='C', requires_grad=False) -> Tensor:
     return Tensor(np.zeros(shape=shape, dtype=dtype, order=order), requires_grad=requires_grad)
 
 
 def zeros_like(a, dtype=None, order='K', subok=True, shape=None, requires_grad=False) -> Tensor:
+    if dtype is None:
+        dtype = a.data.dtype
     return Tensor(np.zeros_like(a=a, dtype=dtype, order=order, subok=subok, shape=shape), requires_grad=requires_grad)
 
 
@@ -240,22 +278,43 @@ def _slice(t: Tensor, idxs) -> Tensor:
     return Tensor(data, requires_grad, dependencies)
 
 
-def pad(t: Tensor, pad_width, mode='constant', **kwargs) -> Tensor:
-    data = np.pad(t.data, pad_width, mode, **kwargs)
-    requires_grad = t.requires_grad
+def concat(tensors: Sequence[Tensor], dim: int = 0) -> Tensor:
+    assert len(tensors) > 0
+    i_dim = tensors[0].dim()
+    data = np.concatenate([t.data for t in tensors], axis=dim)
+    requires_grad = True in [t.requires_grad for t in tensors]
     dependencies: List[Dependency] = []
 
-    if t.requires_grad:
-        def grad_fn(grad: np.ndarray, _) -> np.ndarray:
-            idx = tuple([np.s_[pad_width[i][0]:grad.shape[i] - pad_width[i][1]] for i in range(grad.ndim)])
-            return grad[idx]
+    a = 0
+    b = 0
+    for i, t in enumerate(tensors):
+        b += tensors[i].shape[dim]
+        if t.requires_grad:
+            def grad_fn(grad: np.ndarray, meta) -> np.ndarray:
+                idx = tuple([np.s_[meta["a"]:meta["b"]] if j == dim else np.s_[:] for j in range(i_dim)])
+                return grad[idx]
 
-        dependencies.append(Dependency(t, grad_fn, meta={"name": "pad"}))
+            dependencies.append(Dependency(t, grad_fn, meta={"name": "concat", "id": i, "a": a, "b": b}))
+        a += tensors[i].shape[dim]
 
     return Tensor(data, requires_grad, dependencies)
 
 
-def reshape(t: Tensor, shape) -> Tensor:
+def unsqueeze(x: Tensor, dim: int) -> Tensor:
+    data = np.expand_dims(x.data, axis=dim)
+    requires_grad = x.requires_grad
+    dependencies = []
+
+    if x.requires_grad:
+        def grad_fn(grad: np.ndarray, _) -> np.ndarray:
+            return grad.reshape(x.shape)
+
+        dependencies.append(Dependency(x, grad_fn, meta={"name": "unsqueeze"}))
+
+    return Tensor(data, requires_grad, dependencies)
+
+
+def reshape(t: Tensor, shape: Shapable) -> Tensor:
     data = t.data.reshape(shape)
     requires_grad = t.requires_grad
     dependencies: List[Dependency] = []
@@ -296,56 +355,79 @@ def flatten(t: Tensor, start_dim=0, end_dim=-1) -> Tensor:
     return Tensor(data, requires_grad, dependencies)
 
 
-def concat(tensors: Sequence[Tensor], axis: int = 0) -> Tensor:
-    assert len(tensors) > 0
-    dim = tensors[0].dim()
-    data = np.concatenate([t.data for t in tensors], axis=axis)
-    requires_grad = True in [t.requires_grad for t in tensors]
+def pad(t: Tensor, pad_width, mode='constant', **kwargs) -> Tensor:
+    data = np.pad(t.data, pad_width, mode, **kwargs)
+    requires_grad = t.requires_grad
     dependencies: List[Dependency] = []
 
-    a = 0
-    b = 0
-    for i, t in enumerate(tensors):
-        b += tensors[i].shape[axis]
-        if t.requires_grad:
-            def grad_fn(grad: np.ndarray, meta) -> np.ndarray:
-                idx = tuple([np.s_[meta["a"]:meta["b"]] if j == axis else np.s_[:] for j in range(dim)])
-                return grad[idx]
+    if t.requires_grad:
+        def grad_fn(grad: np.ndarray, _) -> np.ndarray:
+            idx = tuple([np.s_[pad_width[i][0]:grad.shape[i] - pad_width[i][1]] for i in range(grad.ndim)])
+            return grad[idx]
 
-            dependencies.append(Dependency(t, grad_fn, meta={"name": "concat", "id": i, "a": a, "b": b}))
-        a += tensors[i].shape[axis]
+        dependencies.append(Dependency(t, grad_fn, meta={"name": "pad"}))
 
     return Tensor(data, requires_grad, dependencies)
 
 
-def sum(t: Tensor, axis: Optional[Shapable] = None) -> Tensor:
-    """
-    Takes a tensor and return the sum of its components
-    """
-    data = t.data.sum(axis=axis)
+def index_select(t: Tensor, dim: int, index: List[int]) -> Tensor:
+    # TODO: impl grad
+    data = np.take(t.data, index, dim)
     requires_grad = t.requires_grad
     dependencies: List[Dependency] = []
 
     if requires_grad:
-        if axis is None:
+        def grad_fn(grad: np.ndarray, _) -> np.ndarray:
+            pass
+
+        dependencies.append(Dependency(t, grad_fn, meta={"name": "index_select"}))
+
+    return Tensor(data, requires_grad, dependencies)
+
+
+def diag(t: Tensor, diagonal: int = 0) -> Tensor:
+    # TODO: impl grad
+    data = np.diag(t.data, diagonal)
+    requires_grad = t.requires_grad
+    dependencies: List[Dependency] = []
+
+    if requires_grad:
+        def grad_fn(grad: np.ndarray, _) -> np.ndarray:
+            pass
+
+        dependencies.append(Dependency(t, grad_fn, meta={"name": "diag"}))
+
+    return Tensor(data, requires_grad, dependencies)
+
+
+def sum(t: Tensor, dim: Optional[Shapable] = None) -> Tensor:
+    """
+    Takes a tensor and return the sum of its components
+    """
+    data = t.data.sum(axis=dim)
+    requires_grad = t.requires_grad
+    dependencies: List[Dependency] = []
+
+    if requires_grad:
+        if dim is None:
             def grad_fn(grad: np.ndarray, _) -> np.ndarray:
                 return grad * np.ones_like(t.data)
         else:
             def grad_fn(grad: np.ndarray, _) -> np.ndarray:
-                return np.expand_dims(grad, axis=axis) * np.ones_like(t.data)
+                return np.expand_dims(grad, axis=dim) * np.ones_like(t.data)
 
         dependencies.append(Dependency(t, grad_fn, meta={"name": "sum"}))
 
     return Tensor(data, requires_grad, dependencies)
 
 
-def mean(t: Tensor, axis: Optional[Shapable] = None) -> Tensor:
-    data = t.sum(axis=axis)
-    if axis is None:
+def mean(t: Tensor, dim: Optional[Shapable] = None) -> Tensor:
+    data = t.sum(dim=dim)
+    if dim is None:
         numel = t.numel()
     else:
         total_numel = t.numel()
-        numel = total_numel / data.numel()
+        numel = total_numel // data.numel()
     return data / numel
 
 
@@ -434,10 +516,10 @@ def mul(t1: Tensor, t2: Tensor) -> Tensor:
             grad = grad * t2.data
             ndims_added = grad.ndim - t1.data.ndim
             for _ in range(ndims_added):
-                grad = grad.sum(axis=0)
+                grad = grad.sum(dim=0)
             for i, dim in enumerate(t1.shape):
                 if dim == 1:
-                    grad = grad.sum(axis=i, keepdims=True)
+                    grad = grad.sum(dim=i, keepdims=True)
             return grad
 
         dependencies.append(Dependency(t1, grad_fn1, meta={"name": "mul_lhs"}))
@@ -447,10 +529,10 @@ def mul(t1: Tensor, t2: Tensor) -> Tensor:
             grad = grad * t1.data
             ndims_added = grad.ndim - t2.data.ndim
             for _ in range(ndims_added):
-                grad = grad.sum(axis=0)
+                grad = grad.sum(dim=0)
             for i, dim in enumerate(t2.shape):
                 if dim == 1:
-                    grad = grad.sum(axis=i, keepdims=True)
+                    grad = grad.sum(dim=i, keepdims=True)
             return grad
 
         dependencies.append(Dependency(t2, grad_fn2, meta={"name": "mul_rhs"}))
@@ -469,10 +551,10 @@ def truediv(t1: Tensor, t2: Tensor) -> Tensor:
             grad = np.divide(grad, t2.data)
             ndims_added = grad.ndim - t1.data.ndim
             for _ in range(ndims_added):
-                grad = grad.sum(axis=0)
+                grad = grad.sum(dim=0)
             for i, dim in enumerate(t1.shape):
                 if dim == 1:
-                    grad = grad.sum(axis=i, keepdims=True)
+                    grad = grad.sum(dim=i, keepdims=True)
             return grad
 
         dependencies.append(Dependency(t1, grad_fn1, meta={"name": "truediv_lhs"}))
@@ -484,10 +566,10 @@ def truediv(t1: Tensor, t2: Tensor) -> Tensor:
             # grad = np.negative(np.divide(np.multiply(grad, t1.datasets), np.power(t2.datasets, 2)))
             ndims_added = grad.ndim - t2.data.ndim
             for _ in range(ndims_added):
-                grad = grad.sum(axis=0)
+                grad = grad.sum(dim=0)
             for i, dim in enumerate(t2.shape):
                 if dim == 1:
-                    grad = grad.sum(axis=i, keepdims=True)
+                    grad = grad.sum(dim=i, keepdims=True)
             return grad
 
         dependencies.append(Dependency(t2, grad_fn2, meta={"name": "truediv_rhs"}))
@@ -496,7 +578,6 @@ def truediv(t1: Tensor, t2: Tensor) -> Tensor:
 
 
 def matmul(t1: Tensor, t2: Tensor) -> Tensor:
-    # TODO: fix one dim
     """
     if t1 is (n1, m1) and t2 is (m1, m2), then t1 @ t2 is (n1, m2)
     so grad3 is (n1, m2)
@@ -504,33 +585,55 @@ def matmul(t1: Tensor, t2: Tensor) -> Tensor:
         grad1 = grad3 @ t2.T
         grad2 = t1.T @ grad3
     """
-    data = t1.data @ t2.data
+    data = np.matmul(t1.data, t2.data)
     requires_grad = t1.requires_grad or t2.requires_grad
     dependencies: List[Dependency] = []
 
     if t1.requires_grad:
         def grad_fn1(grad: np.ndarray, _) -> np.ndarray:
-            grad = grad @ np.swapaxes(t2.data, -1, -2)
-            ndims_added = grad.ndim - t1.data.ndim
+            data_t1 = t1.data
+            data_t2 = t2.data
+            if t1.dim() == 1:
+                data_t1 = np.expand_dims(data_t1, 0)
+                grad = np.expand_dims(grad, -2)
+            if t2.dim() == 1:
+                data_t2 = np.expand_dims(data_t2, -1)
+                grad = np.expand_dims(grad, -1)
+            data_t2 = np.swapaxes(data_t2, -1, -2)
+            grad_t1 = grad @ data_t2
+            ndims_added = grad_t1.ndim - t1.data.ndim
             for _ in range(ndims_added):
-                grad = grad.sum(axis=0)
+                grad_t1 = grad_t1.sum(axis=0)
             for i, dim in enumerate(t1.shape):
                 if dim == 1:
-                    grad = grad.sum(axis=i, keepdims=True)
-            return grad
+                    grad_t1 = grad_t1.sum(axis=i, keepdims=True)
+            assert grad_t1.shape == t1.shape
+            return grad_t1
 
         dependencies.append(Dependency(t1, grad_fn1, meta={"name": "matmul_lhs"}))
 
     if t2.requires_grad:
         def grad_fn2(grad: np.ndarray, _) -> np.ndarray:
-            grad = np.swapaxes(t1.data, -1, -2) @ grad
-            ndims_added = grad.ndim - t2.data.ndim
+            data_t1 = t1.data
+            data_t2 = t2.data
+            if t1.dim() == 1:
+                data_t1 = np.expand_dims(data_t1, 0)
+                grad = np.expand_dims(grad, -2)
+            if t2.dim() == 1:
+                data_t2 = np.expand_dims(data_t2, -1)
+                grad = np.expand_dims(grad, -1)
+            data_t1 = np.swapaxes(data_t1, -1, -2)
+            grad_t2 = data_t1 @ grad
+            if t2.dim() == 1:
+                grad_t2 = np.squeeze(grad_t2, axis=-1)
+            ndims_added = grad_t2.ndim - t2.data.ndim
             for _ in range(ndims_added):
-                grad = grad.sum(axis=0)
+                grad_t2 = grad_t2.sum(axis=0)
             for i, dim in enumerate(t2.shape):
                 if dim == 1:
-                    grad = grad.sum(axis=i, keepdims=True)
-            return grad
+                    grad_t2 = grad_t2.sum(axis=i, keepdims=True)
+            assert grad_t2.shape == t2.shape
+            return grad_t2
 
         dependencies.append(Dependency(t2, grad_fn2, meta={"name": "matmul_rhs"}))
 
@@ -548,10 +651,10 @@ def power(x: Tensor, e: Tensor) -> Tensor:
             grad = grad * e.data * np.power(x.data, e.data - 1)
             ndims_added = grad.ndim - x.data.ndim
             for _ in range(ndims_added):
-                grad = grad.sum(axis=0)
+                grad = grad.sum(dim=0)
             for i, dim in enumerate(x.shape):
                 if dim == 1:
-                    grad = grad.sum(axis=i, keepdims=True)
+                    grad = grad.sum(dim=i, keepdims=True)
             return grad
 
         dependencies.append(Dependency(x, grad_fn1, meta={"name": "power_lhs"}))
@@ -561,10 +664,10 @@ def power(x: Tensor, e: Tensor) -> Tensor:
             grad = grad * data * np.log(x.data)
             ndims_added = grad.ndim - e.data.ndim
             for _ in range(ndims_added):
-                grad = grad.sum(axis=0)
+                grad = grad.sum(dim=0)
             for i, dim in enumerate(e.shape):
                 if dim == 1:
-                    grad = grad.sum(axis=i, keepdims=True)
+                    grad = grad.sum(dim=i, keepdims=True)
             return grad
 
         dependencies.append(Dependency(e, grad_fn2, meta={"name": "power_rhs"}))
@@ -596,14 +699,14 @@ def transpose(a: Tensor, dim1: int, dim2: int) -> Tensor:
     return Tensor(data, requires_grad, dependencies)
 
 
-def permute(a: Tensor, axes: Union[List, Tuple]) -> Tensor:
-    data = a.data.transpose(axes)
+def permute(a: Tensor, dims: Union[List, Tuple]) -> Tensor:
+    data = a.data.transpose(dims)
     requires_grad = a.requires_grad
     dependencies: List[Dependency] = []
 
     if a.requires_grad:
         def grad_fn(grad: np.ndarray, _) -> np.ndarray:
-            axes_t = invert_permutation(axes)
+            axes_t = invert_permutation(dims)
             return grad.transpose(axes_t)
 
         dependencies.append(Dependency(a, grad_fn, meta={"name": "permute"}))
