@@ -1,13 +1,10 @@
+from ..autograd.tensor import Shapable
 from typing import (
-    Tuple,
     Union,
-    List,
 )
 
 import numpy as np
 
-Shapable = Union[Tuple, List]
-
 
 # TODO: impl initializers
 # TODO: impl with tensor.zero_grad()
